create table follow_clue
(
    id          bigint auto_increment comment 'id',
    vin         varchar(128) null comment '车架号 有档案是非空',
    plate_no    varchar(32)  null comment '车牌号 事故车时非空',
    clue_state  int(3)       not null comment '线索状态 1: 等待开始 2：进行中 3：成交 4：战败',
    clue_type   int(3)       not null comment '线索类型 1：首保 2：流失预警 3：事故车 4：续保',
    start_time  datetime     not null comment '线索开始跟进时间',
    end_time    datetime     not null comment '线索结束跟进时间',
    close_time  datetime     null comment '终止时间',
    group_id    bigint       not null comment '集团id',
    create_time datetime     not null,
    update_time datetime     null,
    constraint follow_clue_pk
        primary key (id)
)
    comment '跟进线索';

create index follow_clue_plate_no_index
    on follow_clue (plate_no);

create index follow_clue_vin_index
    on follow_clue (vin);

create table clue_task
(
    id               bigint auto_increment
        primary key,
    clue_id          bigint      not null comment '跟进线索id',
    type             int(3)      not null comment '设置类型 1：首保  2：流失预警 3：事故车 4：续保 等',
    begin_time       datetime    null comment '任务开始时间',
    redistribution   tinyint(1)  not null comment '二次分配',
    deadline         datetime    not null comment '任务截止时间',
    state            int(3)      not null comment '完成状态 1：进行中 2：已完成 3：已战败',
    close_time       datetime    null comment '任务终止时间',
    follow_user      bigint      not null comment '当前跟进人',
    follow_user_name varchar(64) null comment '当前跟进人名称',
    follow_shop      bigint      not null comment '当前跟进服务站',
    finish_user      bigint      null comment '完成人',
    finish_user_name varchar(64) null comment '完成人名称',
    finish_shop      bigint      null comment '完成门店',
    group_id         bigint      not null comment '集团id',
    create_time      datetime    not null,
    update_time      datetime    null
)
    comment '线索任务';

create index clue_task_clue_id_index
    on clue_task (clue_id);
create index clue_task_finish_user_finish_shop_index
    on clue_task (finish_user, finish_shop);

create index clue_task_follow_user_follow_shop_index
    on clue_task (follow_user, follow_shop);

alter table clue_task
    add reason int(3) null comment '战败原因 ' after finish_shop;

alter table original_data
    modify customer_id bigint null comment '客户id ';

alter table original_data
    add frame_no varchar(128) null comment '车架号' after plate_no;

create index original_data_frame_no_index
    on original_data (frame_no);

alter table clue_task
    add rpc_success tinyint(1) null comment '状态完结 rpc调用成功' after group_id;

alter table follow_clue
    add suggest_shop_id bigint null comment '推荐门店id' after start_time;
alter table follow_clue
    add suggest_mobile varchar(64) null comment '推荐联系方式' after suggest_shop_id;
alter table follow_clue
    add next_time datetime null comment '下次跟进时间' after start_time;

alter table follow_clue
    add origin_time datetime not null comment '初始时间' after vin;

alter table follow_clue
    add pre_record_id bigint null comment '上次跟进待办id' after next_time;

alter table fw_valhalla.follow_clue
    add biz_id bigint not null comment '业务数据的原始id' after end_time;

create index follow_clue_biz_id_index
    on fw_valhalla.follow_clue (biz_id);

alter table fw_valhalla.clue_task
    add times int(5) null comment '已跟进次数' after reason;

