package cn.fw.valhalla.rpc.shirasawa;

import cn.fw.data.base.domain.common.Message;
import cn.fw.shirasawa.sdk.api.FollowApiService;
import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.sdk.param.FollowGenerateDTO;
import cn.fw.shirasawa.sdk.param.NewRecordDTO;
import cn.fw.shirasawa.sdk.param.TaskCompleteDTO;
import cn.fw.shirasawa.sdk.param.TerminationDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.ClueStopDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.FollowInitDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.NextRecordDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 跟进系统rpc
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ShirasawaRpcService
 * @description : 跟进系统rpc
 * @date : 2022-07-22 14:50
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ShirasawaRpcService {

    private final FollowApiService followApiService;

    /**
     * 完成/终止跟进任务
     *
     * @param dto
     */
    public boolean stopTask(final ClueStopDTO dto) {
        try {
            if (Objects.isNull(dto.getType())) {
                return true;
            }
            if (!dto.isTermination()) {
                TaskCompleteDTO completeDTO = new TaskCompleteDTO();
                BeanUtils.copyProperties(dto, completeDTO);
                completeDTO.setType(dto.getType());
                completeDTO.setBusinessType(dto.getBusinessType());
                Message<Void> completeMsg = followApiService.completeTask(completeDTO);
                log.info("followApiService.completeTask: msg.code={}, msg.result={}", completeMsg.getCode(), completeMsg.getResult());
                return 0 == completeMsg.getCode();
            }
            TerminationDTO terminationDTO = new TerminationDTO();
            BeanUtils.copyProperties(dto, terminationDTO);
            terminationDTO.setType(dto.getType());
            terminationDTO.setBusinessType(dto.getBusinessType());
            Message<Void> terminationMsg = followApiService.terminationTask(terminationDTO);
            log.info("followApiService.terminationTask: msg.code={}, msg.result={}", terminationMsg.getCode(), terminationMsg.getResult());
            return 0 == terminationMsg.getCode();
        } catch (Exception e) {
            log.error("停止跟进任务失败。detailId: [{}]", dto.getDetailId(), e);
        }
        return false;
    }

    /**
     * 创建跟进系统原始数据
     *
     * @param dto
     */
    public void createFollowData(final FollowInitDTO dto) {
        FollowGenerateDTO followGenerateDTO = new FollowGenerateDTO();
        BeanUtils.copyProperties(dto, followGenerateDTO);
        followGenerateDTO.setType(dto.getType());
        followGenerateDTO.setBusinessType(dto.getBusinessType());
        followGenerateDTO.setNoteMap(dto.getNoteMap());
        final Message<Void> msg = followApiService.generate(followGenerateDTO);
        BV.isTrue(msg.isSuccess(), msg::getResult);
    }

    /**
     * 完成跟进
     *
     * @param recordId
     * @param userId
     */
    public void completeRecordTodo(Long recordId, Long userId) {
        followApiService.completeFollow(recordId, userId);
    }


    /**
     * 创建下次待办
     *
     * @param dto
     */
    public void createNextFollow(NextRecordDTO dto) {
        NewRecordDTO newRecordDTO = new NewRecordDTO();
        BeanUtils.copyProperties(dto, newRecordDTO);
        newRecordDTO.setBusinessType(BusinessTypeEnum.AS.getValue());
        newRecordDTO.setDataType(DataTypeEnum.ofValue(dto.getType()));
        final Message<Boolean> msg = followApiService.createNextRecord(newRecordDTO);
        BV.isTrue(msg.isSuccess(), msg::getResult);
    }
}
