package cn.fw.valhalla.component;

import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.sdk.mq.FollowResultDTO;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.hutool.core.collection.ListUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Objects;
import java.util.Set;

/**
 * @author : kurisu
 * Date: 2020/9/16
 * Time: 10:57
 * Description:
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = FollowResultDTO.FOLLOW_TOPIC, consumerGroup = "${spring.application.name}_follow_todo_result")
public class FollowResultConsumer implements RocketMQListener<FollowResultDTO> {

    private final FollowBizService followBizService;
    private final ClueTaskService clueTaskService;


    @Autowired
    public FollowResultConsumer(final FollowBizService followBizService,
                                final ClueTaskService clueTaskService) {
        this.followBizService = followBizService;
        this.clueTaskService = clueTaskService;
    }

    @Override
    public void onMessage(FollowResultDTO dto) {
        log.info("处理跟进结果MQ消息：{}", JSONObject.toJSONString(dto));
        try {
            LinkedList<Integer> linkedList = ListUtil.toLinkedList(
                    DataTypeEnum.AC.getValue(),
                    DataTypeEnum.FM.getValue(),
                    DataTypeEnum.RM.getValue(),
                    DataTypeEnum.IR.getValue(),
                    DataTypeEnum.PL.getValue()
            );
            Set<Integer> validSet = new HashSet<>(linkedList);
            if (!validSet.contains(dto.getType())) {
                return;
            }
            Long val = Long.valueOf(dto.getDetailId());
            ClueTask task = clueTaskService.getById(val);
            if (Objects.isNull(task)) {
                return;
            }
            followBizService.afterFollowComplete(task, dto.getRecordId(), dto.getResultTime(), dto.isOverdue());
        } catch (Exception e) {
            log.error("处理跟进结果MQ异常", e);
            throw e;
        }
    }
}
