package cn.fw.valhalla.service.bus.follow;

import cn.fw.valhalla.domain.db.customer.AccidentPool;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.dto.ShouldBeCompletedDTO;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.query.BeCompletedQuery;
import cn.fw.valhalla.sdk.param.ShouldBeCompletedQuery;
import cn.fw.valhalla.sdk.result.ShouldBeCompletedResult;
import cn.fw.valhalla.service.data.AccidentPoolService;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.fw.valhalla.service.data.FollowClueService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 跟进api接口业务实现
 *
 * @author : kurisu
 * @version : 1.0
 * @className : FollowApiBizService
 * @description : 跟进api接口业务实现
 * @date : 2022-10-20 14:35
 */
@Service
public class FollowApiBizService {
    private final ClueTaskService clueTaskService;
    private final AccidentPoolService accidentPoolService;
    private final FollowClueService followClueService;

    @Autowired
    public FollowApiBizService(final ClueTaskService clueTaskService,
                               final AccidentPoolService accidentPoolService,
                               final FollowClueService followClueService) {
        this.clueTaskService = clueTaskService;
        this.accidentPoolService = accidentPoolService;
        this.followClueService = followClueService;
    }

    /**
     * 查询应成交台数
     *
     * @param query
     * @param followTypeEnum
     * @return
     */
    public List<ShouldBeCompletedResult> queryShouldBeCompleted(ShouldBeCompletedQuery query, FollowTypeEnum followTypeEnum) {
        BeCompletedQuery search = new BeCompletedQuery();
        search.setType(followTypeEnum.getValue());
        search.setShopId(query.getShopId());
        search.setUserId(query.getUserId());
        search.setStartTime(query.getStartTime());
        search.setEndTime(query.getEndTime());
        List<ShouldBeCompletedDTO> dtoList = clueTaskService.queryShouldBeCompleted(search);
        List<ShouldBeCompletedResult> results = new ArrayList<>();
        for (ShouldBeCompletedDTO dto : dtoList) {
            ShouldBeCompletedResult result = new ShouldBeCompletedResult();
            result.setVin(dto.getVin());
            result.setUserId(dto.getUserId());
            result.setUserName(dto.getUserName());
            result.setShopId(dto.getShopId());
            results.add(result);
        }
        return results;
    }

    public ClueTask queryAccidentFollowerByPlate(String plateNo, Long groupId) {
        AccidentPool lastOne = accidentPoolService.getOne(Wrappers.<AccidentPool>lambdaQuery()
                .eq(AccidentPool::getPlateNo, plateNo)
                .eq(AccidentPool::getGroupId, groupId)
                .orderByDesc(AccidentPool::getCreateTime)
                .last(" limit 1 ")
        );

        if (Objects.isNull(lastOne)) {
            return null;
        }
        FollowClue lastOngoingClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getPlateNo, plateNo)
                .eq(FollowClue::getClueType, FollowTypeEnum.AC)
                .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                .eq(FollowClue::getGroupId, groupId)
                .last(" limit 1 ")
        );

        if (Objects.isNull(lastOngoingClue)) {
            return null;
        }
        return clueTaskService.queryOngoingTaskByClueId(lastOngoingClue.getId());
    }
}
